"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cryptrack = void 0;
const elliptic_1 = require("elliptic");
const crypto_js_1 = require("crypto-js");
const jose_1 = require("jose");
class Cryptrack {
    constructor() {
        this._curve = new elliptic_1.ec('secp256k1');
        this._keys = this._curve.genKeyPair();
        this._store = new Map();
        this._jwtKeyPromise = (0, jose_1.generateKeyPair)('HS256').then(kp => kp.privateKey);
    }
    register(id, publicKeyHex) {
        try {
            const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
            const sharedSecret = this._keys.derive(peerPubKey).toString(16);
            const sharedKey = (0, crypto_js_1.SHA256)(sharedSecret).toString();
            this._store.set(id, sharedKey);
            return sharedKey;
        }
        catch (err) {
            throw new Error(`Invalid public key: ${err.message}`);
        }
    }
    getKey(id) {
        return this._store.get(id);
    }
    encrypt(id, data) {
        const key = this.getKey(id);
        if (!key)
            throw new Error(`Shared key for ID "${id}" not found`);
        return crypto_js_1.AES.encrypt(data, key).toString();
    }
    decrypt(id, encrypted) {
        const key = this.getKey(id);
        if (!key)
            throw new Error(`Shared key for ID "${id}" not found`);
        try {
            const decrypted = crypto_js_1.AES.decrypt(encrypted, key).toString(crypto_js_1.enc.Utf8);
            if (!decrypted)
                throw new Error('Failed to decode decrypted data');
            return decrypted;
        }
        catch (err) {
            throw new Error(`Decryption failed: ${err.message}`);
        }
    }
    getPublicKey() {
        return this._keys.getPublic('hex');
    }
    signToken(id_1, payload_1) {
        return __awaiter(this, arguments, void 0, function* (id, payload, expires = '1h') {
            const key = yield this._jwtKeyPromise;
            return new jose_1.SignJWT(payload)
                .setProtectedHeader({ alg: 'HS256' })
                .setSubject(id)
                .setIssuedAt()
                .setExpirationTime(expires)
                .sign(key);
        });
    }
    verifyToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = yield this._jwtKeyPromise;
            const { payload } = yield (0, jose_1.jwtVerify)(token, key);
            if (!payload.sub)
                throw new Error('Token missing subject');
            return {
                id: payload.sub,
                payload,
            };
        });
    }
}
exports.Cryptrack = Cryptrack;
